@@ -86,49 +86,6 @@ ColorTup = namedtuple('ColorTup', ['r', 'g', 'b'])
 AppInfoTup = namedtuple('AppInfoTup', ['app', 'pid', 'windows'])
 
 
-def im_get_comp_color(filename):
-    """Find the complimentary colour of the average colour of an image.
-
-    Uses ImageMagick to read and process the image
-
-    Args:
-        filename : the filename of the image
-
-    Returns:
-        a tuple of r,g,b values (0-255)
-
-    """
-
-    cmdstr = "convert " + filename + " -colors 16 -depth 8 -format ""%c"" " + \
-             "histogram:info:|sort -rn|head -n 1| grep -oe '#[^\s]*'"
-    cmd = subprocess.Popen(cmdstr, shell=True, stdout=subprocess.PIPE)
-
-    for line in cmd.stdout:
-        pass
-
-    ll1 = str(line)
-    astr = ll1[2:9]
-
-    cmdstr = "convert xc:'" + astr + "' -modulate 100,100,0 -depth 8 txt:"
-    cmd = subprocess.Popen(cmdstr, shell=True, stdout=subprocess.PIPE)
-
-    for line in cmd.stdout:
-        pass
-
-    l1p = str(line)
-    lll = l1p.split(" ")
-    astr = lll
-    astr = lll[3].lstrip("#")
-    if len(astr) == 8:
-        # remove alpha channel
-        astr = astr[0:6]
-
-    red = int(astr[0:2], 16)
-    green = int(astr[2:4], 16)
-    blue = int(astr[4:6], 16)
-    return red, green, blue
-
-
 def get_avg_color(pixbuf):
     """calculate the average colour of a pixbuf.
 
@@ -220,6 +177,7 @@ class PulseTimer(object):
             once_only : do only one iteration of pulsing, then stop
         """
 
+        self.timer_count = 0
         self.app = app
         self.app.pulse_step = 0
         self.app.is_pulsing = True
@@ -237,6 +195,13 @@ class PulseTimer(object):
         Redraw the app's icon
         """
 
+        self.timer_count += 1
+        if self.timer_count / int(1000/CONST_PULSE_DELAY) == 45:
+            # we've been pulsing for long enough, the user will be getting a headache
+            self.remove_timer()
+            self.app.queue_draw()
+            return False
+
         if self.app.pulse_step != CONST_PULSE_STEPS:
             self.app.pulse_step += 1
         else:
@@ -245,18 +210,25 @@ class PulseTimer(object):
             if (self.app.startup_id is not None) and not self.once_only:
                 self.app.pulse_step = 0
             else:
-                # we only want the icon to pulse one
+                # we only want the icon to pulse once
                 # if we have a startup_id the notification needs to be cancelled
                 if self.app.startup_id is not None:
                     self.app.cancel_startup_notification()
 
-                self.app.is_pulsing = False
-                GObject.source_remove(self.timer_id)
+                self.remove_timer()
 
         self.app.queue_draw()
-
         return True
 
+    def remove_timer(self):
+        """
+            Cancel the timer and stop the app icon pulsing...
+            
+        """
+
+        self.app.is_pulsing = False
+        GObject.source_remove(self.timer_id)
+
 CONST_BLINK_DELAY = 330
 
 
@@ -459,7 +431,7 @@ class DockedApp(object):
 
         return ret_val
 
-    def setup_from_wnck(self, wnck_app, wnck_class, app_match_list):
+    def setup_from_wnck(self, wnck_app, wnck_class, app_match_list, win_ws_changed_cb):
         """Set up for an already running app, obtaining info from libWnck.
 
         Will attempt to get the app path, icon path, .desktop file, currently
@@ -470,6 +442,7 @@ class DockedApp(object):
             wnck_class  : a WnckClass obect relating to the app
             app_match_list : a list of tuples containing details of apps which
                              are hard to match with their .desktop files
+            wnck_ws_changed_cb : workspace changed callback
 
         Returns: True if successful, False otherwise
         """
@@ -492,6 +465,7 @@ class DockedApp(object):
         # get the currently open windows
         for win in wnck_app.get_windows():
             win.connect("state-changed", self.win_state_changed)
+            win.connect("workspace-changed", win_ws_changed_cb)
             win_class = win.get_class_group()
             win_class_name = win_class.get_res_class()
             # if self.wm_class_group_name == "":
@@ -1169,7 +1143,7 @@ class DockedApp(object):
             # draw the icon semi-transparently according to how far through the
             # animation we are
 
-            half_way = CONST_PULSE_STEPS/2
+            half_way = int(CONST_PULSE_STEPS/2)
             if self.pulse_step <= half_way:
                 alpha = 1.0 - (self.pulse_step / half_way)
             else:
@@ -1369,10 +1343,7 @@ class DockedApp(object):
         """
 
         def rounded_rectangle(cr, x, y, w, h, r=20):
-            """ Convenience function to draw a rounded rectangle, really needs
-                to be in a library module somewhere, as its used bythe window
-                list
-
+            """ Convenience function to draw a rounded rectangle
                 # Attribution:
                 # https://stackoverflow.com/questions/2384374/rounded-rectangle-in-pygtk
                 # This is just one of the samples from
@@ -1417,36 +1388,51 @@ class DockedApp(object):
         height = 8              # total height of the progress bar
         line_width = 2          # border line width
         int_height = height - line_width * 2  # interior height
-        left = 8
+        left = 8.5
         width = draw_size - left * 2   # width of the progress bar
 
-        top = (draw_size/8) * 5
+        top = (draw_size/8) * 5 +0.5
 
         ctx.save()
         ctx.scale(self.drawing_area_size/draw_size, self.drawing_area_size/draw_size)
 
+        # fill the interior with the background colour
+        ctx.set_line_width((1))
+        ctx.set_source_rgb(bk_red, bk_green, bk_blue)
+        rounded_rectangle(ctx, left, top, width, height, 7)
+        ctx.stroke_preserve()
+        ctx.fill()
+
+        # fill part of the interior with a different colour, depending on
+        # the progress value
+        ctx.set_source_rgb(fred, fgreen, fblue)
+        rounded_rectangle(ctx, left + line_width-1, top,
+                         (width - (line_width-1)*2) * self.progress_val,
+                          height, 7)
+        ctx.fill()
+
         # draw exterior of the progress bar
         ctx.set_source_rgb(brd_red, brd_green, brd_blue)
         ctx.set_line_width(2)
         rounded_rectangle(ctx, left, top, width, height, 7)
         ctx.stroke()
 
-        # fill the interior with the background colour
-        ctx.set_line_width(1)
-        ctx.set_source_rgb(bk_red, bk_green, bk_blue)
-        rounded_rectangle(ctx, left + line_width, top + line_width,
-                          width - line_width*2, height-line_width*2,
-                          2)
 
-        ctx.fill()
+        #ctx.set_line_width(1)
+        #ctx.set_source_rgb(bk_red, bk_green, bk_blue)
+        #rounded_rectangle(ctx, left + line_width, top + line_width,
+        #                  width - line_width*2, height-line_width*2,
+         #                 2)
+
+        #ctx.fill()
 
         # fill part of the interior with a different colour, depending on
         # the progress value
-        ctx.set_source_rgb(fred, fgreen, fblue)
-        rounded_rectangle(ctx, left + line_width, top + line_width,
-                          (width - line_width*2) * self.progress_val,
-                          height-line_width*2, 2)
-        ctx.fill()
+        #ctx.set_source_rgb(fred, fgreen, fblue)
+        #rounded_rectangle(ctx, left + line_width, top + line_width,
+        #                  (width - line_width*2) * self.progress_val,
+        #                  height-line_width*2, 2)
+        #ctx.fill()
         ctx.restore()
 
     def draw_attention_badge(self, ctx):
